/*global PouchDB: true, couchEval: true, buildProvidesCtx: true */

"use strict";

//fixme
var COUCHDB_HOST = "http://localhost:5984";

(function () {
  function buildProvidesCtx() {
    var providesFuncs = {};
    return {
      provides: function provides(type, func) {
        providesFuncs[type] = function () {
          return {
            code: 200,
            body: func()
          };
        };
      },
      funcs: providesFuncs
    };
  }

  function Show(db) {
    function show(showPath, callback) {
      var designDocName = showPath.split("/")[0];
      var showName = showPath.split("/")[1];
      var docId = showPath.split("/")[2];
      db.get("_design/" + designDocName, function (err, designDoc) {
        if (err) {
          callback(err, null);
          return;
        }
        db.get(docId, function (err, doc) {
          //doc might be null - that's ok and expected.
          var result;
          var providesCtx = buildProvidesCtx();
          try {
            var func = couchEval(designDoc, {
              provides: providesCtx.provides,
              registerTypes: function () {}
            }, designDoc.shows[showName]);
            result = func(doc, null);
          } catch (e) {
            callback({"exception_occurred": e}, null);
          }

          if (!result) {
            result = providesCtx.funcs.html();
          }
          callback(null, result);
        });
      });
    }

    return {
      show: show
    };
  }

  function List(db) {
    function list(listPath, opts, callback) {
      if (typeof opts === "function") {
        callback = opts;
        opts = {};
      }
      var designDocName = listPath.split("/")[0];
      var listName = listPath.split("/")[1];
      var viewName = listPath.split("/")[2];

      db.get("_design/" + designDocName, function (err, designDoc) {
        if (err) {
          callback(err, null);
          return;
        }
        db.query(designDocName + "/" + viewName, opts, function (err, view) {
          if (err) {
            callback(err, null);
            return;
          }
          var result;
          var sent = "";
          var providesCtx = buildProvidesCtx();
          var i = -1;
          try {
            var func = couchEval(designDoc, {
              provides: providesCtx.provides,
              registerTypes: function () {},
              getRow: function () {
                i += 1;
                return view.rows[i];
              },
              start: function () {},
              send: function (data) {
                sent += data;
              },
            }, designDoc.lists[listName]);
            //TODO: (head, req) arguments should make more sense.
            result = func(null, {
              info: {
                db_name: "shared_lists_test"
              },
              headers: {
                Host: COUCHDB_HOST.split("/")[2]
              },
              raw_path: "/shared_lists_test/_design/" + designDocName + "/_list/" + listName + "/" + viewName
            }) || {};
          } catch (e) {
            callback({"exception_occurred": e}, null);
            return;
          }

          result.body = sent + (result.body || "");
          if (!result) {
            //FIXME. Same for show()
            result = providesCtx.funcs.atom();
            result.body = sent + result.body;
          }
          callback(null, result);
        });
      });
    }

    return {
      list: list
    };
  }

  function Update(db) {
    return {
      updating: {
        put: function (doc, opts, callback) {
          if (opts.validation) {
            db.validating.put();
          } else {
            db.put();
          }
        },
        post: function (doc, opts, callback) {
          if (opts.validation) {
            db.validating.post();
          } else {
            db.post();
          }
        }
      }
    }
  }
  //no need to track deletes
  Show._delete = function () {};
  List._delete = function () {};
  Update._delete = function () {};

  PouchDB.plugin('show', Show);
  PouchDB.plugin('list', List);
  PouchDB.plugin('update', Update);
}());
